#include <algorithm>
#include <stdexcept>
#include <msp/fs/dir.h>
#include <msp/graphics/display_private.h>
#include <msp/graphics/window_private.h>
#include "xineengine.h"
#include "xinestream.h"

using namespace std;
using namespace Msp;

XineEngine::XineEngine(Graphics::Window &w):
	window(w),
	pending_expose(0)
{
	engine = xine_new();

	FS::Path config_fn = FS::get_home_dir()/".xine"/"config";
	xine_config_load(engine, config_fn.c_str());

	xine_init(engine);

	audio_driver = xine_open_audio_driver(engine, "auto", 0);
	if(!audio_driver)
		throw runtime_error("Could not open audio driver");

	visual.display = window.get_display().get_private().display;
	visual.screen = 0;
	visual.d = window.get_private().window;
	visual.user_data = this;
	visual.dest_size_cb = &dest_size_cb;
	visual.frame_output_cb = &frame_output_cb;
	visual.lock_display = 0;
	visual.unlock_display = 0;

	video_driver = xine_open_video_driver(engine, "auto", XINE_VISUAL_TYPE_X11, &visual);
	if(!video_driver)
		throw runtime_error("Could not open video driver");

	window.signal_expose.connect(sigc::mem_fun(this, &XineEngine::window_exposed));
}

XineEngine::~XineEngine()
{
	xine_close_video_driver(engine, video_driver);
	xine_close_audio_driver(engine, audio_driver);
	xine_exit(engine);
}

void XineEngine::add_stream(XineStream &stream)
{
	streams.push_back(&stream);
}

void XineEngine::remove_stream(XineStream &stream)
{
	list<XineStream *>::iterator i = find(streams.begin(), streams.end(), &stream);
	if(i!=streams.end())
		streams.erase(i);
}

void XineEngine::tick()
{
	if(pending_expose)
	{
		xine_port_send_gui_data(video_driver, XINE_GUI_SEND_EXPOSE_EVENT, &pending_expose->xevent);
		delete pending_expose;
		pending_expose = 0;
	}

	for(list<XineStream *>::const_iterator i=streams.begin(); i!=streams.end(); ++i)
		(*i)->tick();
}

void XineEngine::window_exposed(unsigned, unsigned, unsigned, unsigned, const Graphics::Window::Event &event)
{
	delete pending_expose;
	pending_expose = new Graphics::Window::Event(event);
}

void XineEngine::dest_size_cb(void *user_data, int, int, double, int *dest_width, int *dest_height, double *dest_pixel_aspect)
{
	XineEngine &engine = *reinterpret_cast<XineEngine *>(user_data);
	*dest_width = engine.window.get_width();
	*dest_height = engine.window.get_height();
	*dest_pixel_aspect = 1.0;
}

void XineEngine::frame_output_cb(void *user_data, int, int, double, int *dest_x, int *dest_y, int *dest_width, int *dest_height, double *dest_pixel_aspect, int *win_x, int *win_y)
{
	XineEngine &engine = *reinterpret_cast<XineEngine *>(user_data);
	*dest_x = 0;
	*dest_y = 0;
	*dest_width = engine.window.get_width();
	*dest_height = engine.window.get_height();
	*dest_pixel_aspect = 1.0;
	*win_x = 0;
	*win_y = 0;
}
